!
! Copyright (C) 2017 Mitsuaki Kawamura
! This file is distributed under the terms of the
! GNU General Public License. See the file `License'
! in the root directory of the present distribution,
! or http://www.gnu.org/copyleft/gpl.txt .
!
!>
!> Compute @f$\lamba_{n k}@f$ and \mu_{n k} on dense grid
!>
PROGRAM lambda_mu_k
  !
  USE kinds, ONLY : DP
  USE mp_world, ONLY : mpime
  USE io_global, ONLY : stdout
  USE fermisurfer_common, ONLY : b_low, b_high, write_fermisurfer
  USE environment,ONLY : environment_start, environment_end
  USE mp_global,  ONLY : mp_startup, mp_global_end
  USE wvfct, ONLY : et
  USE klist, ONLY : nks
  USE ener, ONLY : ef
  !
  USE sctk_val, ONLY : dltf, Zf
  !
  USE sctk_stdin, ONLY : stdin_scdft, stdin_control
  USE sctk_read_files, ONLY : read_elph, read_Coulomb
  USE sctk_ini_delta, ONLY : ini_lambda_mu
  USE sctk_gapeq_rhs, ONLY : make_lambda_mu_f
  USE sctk_rotate_kernel, ONLY : expand_g_v_f
  !
  IMPLICIT NONE
  !
  REAL(dp),ALLOCATABLE :: eig(:,:)
  !
#if defined(__MPI)
  CALL mp_startup ( )
#endif
  CALL environment_start ( 'LAMBDA_MU_K' )
  !
  WRITE(stdout,*)
  WRITE(stdout,*) "#####  Read from data-file.xml  #####"
  WRITE(stdout,*)
  CALL stdin_control()
  CALL read_xml_file ( )
  CALL read_a2Fsave()
  !
  WRITE(stdout,*)
  WRITE(stdout,*) "#####  Read from STDIN  #####"
  WRITE(stdout,*)
  CALL stdin_scdft()
  !
  WRITE(stdout,*)
  WRITE(stdout,*) "#####  Read from elph*.dat  #####"
  WRITE(stdout,*)
  CALL read_elph()
  !
  WRITE(stdout,*)
  WRITE(stdout,*) "#####  Read from vc*.dat  #####"
  WRITE(stdout,*)
  CALL read_Coulomb()
  !
  WRITE(stdout,*)
  WRITE(stdout,*) "#####  Average matrix in grid  #####"
  WRITE(stdout,*)
  CALL expand_g_v_f()
  !
  WRITE(stdout,*)
  WRITE(stdout,*) "#####  Set or read initial delta  #####"
  WRITE(stdout,*)
  CALL ini_lambda_mu()
  !
  WRITE(stdout,*)
  WRITE(stdout,*) "#####  Integrate gapeq  #####"
  WRITE(stdout,*)
  CALL make_lambda_mu_f()
  !
  WRITE(stdout,*)
  WRITE(stdout,*) "#####  Write delta.fs  #####"
  WRITE(stdout,*)
  IF(mpime == 0) THEN
     ALLOCATE(eig(b_low:b_high,nks))
     eig(b_low:b_high,1:nks) = et(b_low:b_high,1:nks) - ef
     CALL write_fermisurfer(eig, dltF, "mu.frmsf")
     CALL write_fermisurfer(eig, ZF, "lambda.frmsf")
     DEALLOCATE(eig)
  END IF
  !
  CALL environment_end ( 'LAMBDA_MU_K' )
#if defined(__MPI)
  CALL mp_global_end ( )
#endif
  !
END PROGRAM lambda_mu_k

